@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2022 Olivier Friard
+Copyright 2012-2023 Olivier Friard
 
 This file is part of BORIS.
 
@@ -280,7 +280,12 @@ class Video_overlay_dialog(QDialog):
 
 class Input_dialog(QDialog):
     """
-    dialog for user input. Elements can be checkbox, lineedit or spinbox
+    dialog for user input. Elements can be:
+        checkbox (cb): Tuple(str, str, bool)
+        lineedit (le): Tuple(str, str)
+        spinbox (sp)
+        items list (il)
+
     """
 
     def __init__(self, label_caption: str, elements_list: list, title: str = ""):
@@ -293,7 +298,7 @@ class Input_dialog(QDialog):
         self.label.setText(label_caption)
         hbox.addWidget(self.label)
 
-        self.elements = {}
+        self.elements: dict = {}
         for element in elements_list:
 
             if element[0] == "cb":  # checkbox
@@ -335,8 +340,7 @@ class Input_dialog(QDialog):
         self.pbCancel.clicked.connect(self.reject)
         hbox2.addWidget(self.pbCancel)
 
-        self.pbOK = QPushButton(cfg.OK)
-        self.pbOK.clicked.connect(self.accept)
+        self.pbOK = QPushButton(cfg.OK, clicked=self.accept)
         self.pbOK.setDefault(True)
         hbox2.addWidget(self.pbOK)
 
@@ -626,6 +630,7 @@ class FindReplaceEvents(QWidget):
         self.clickSignal.emit(msg)
 
 
+'''
 class explore_project_dialog(QDialog):
     """
     "explore project" dialog box
@@ -638,6 +643,8 @@ class explore_project_dialog(QDialog):
 
         hbox = QVBoxLayout()
 
+        hbox.addWidget(QLabel("Search in all observations"))
+
         self.lb_subject = QLabel("Subject")
         hbox.addWidget(self.lb_subject)
         self.find_subject = QLineEdit()
@@ -675,6 +682,7 @@ class explore_project_dialog(QDialog):
         hbox.addLayout(hbox2)
 
         self.setLayout(hbox)
+'''
 
 
 class Results_dialog(QDialog):
@@ -800,10 +808,7 @@ class View_explore_project_results(QWidget):
         vbox.addWidget(self.tw)
 
         hbox2 = QHBoxLayout()
-
-        self.pbOK = QPushButton("OK")
-        self.pbOK.clicked.connect(self.close)
-        hbox2.addWidget(self.pbOK)
+        hbox2.addWidget(QPushButton("OK", clicked=self.close))
 
         vbox.addLayout(hbox2)
 
