@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2022 Olivier Friard
+Copyright 2012-2023 Olivier Friard
 
 
   This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@ Copyright 2012-2022 Olivier Friard
 
 import logging
 import os
-import pathlib
+import pathlib as pl
 import subprocess
 from decimal import Decimal as dec
 
@@ -85,6 +85,9 @@ def events_snapshots(self):
         n_observations=len(selected_observations),
     )
 
+    if parameters == {}:
+        return
+
     if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
         return
 
@@ -228,25 +231,6 @@ def events_snapshots(self):
                             )
 
                             vframes = 1 if not time_interval else int(mediafile_fps * time_interval * 2)
-                            ffmpeg_command = (
-                                f'"{self.ffmpeg_bin}" '
-                                f"-ss {start:.3f} "
-                                f'-i "{media_path}" '
-                                f"-vframes {vframes} "
-                                f'"{exportDir}{os.sep}'
-                                f"{util.safeFileName(obsId)}"
-                                f"_PLAYER{nplayer}"
-                                f"_{util.safeFileName(subject)}"
-                                f"_{util.safeFileName(behavior)}"
-                                f'_{start:.3f}_%08d.{self.frame_bitmap_format.lower()}"'
-                            )
-
-                            logging.debug(f"ffmpeg command: {ffmpeg_command}")
-
-                            p = subprocess.Popen(
-                                ffmpeg_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True
-                            )
-                            out, error = p.communicate()
 
                         if cfg.STATE in behavior_state:
                             if idx % 2 == 0:
@@ -301,26 +285,28 @@ def events_snapshots(self):
                                     self.projectFileName,
                                 )
 
-                                extension = "png"
                                 vframes = int((stop - start) * mediafile_fps + time_interval * mediafile_fps * 2)
-                                ffmpeg_command = (
-                                    f'"{self.ffmpeg_bin}" -ss {start:.3f} '
-                                    f'-i "{media_path}" '
-                                    f"-vframes {vframes} "
-                                    f'"{exportDir}{os.sep}'
-                                    f"{util.safeFileName(obsId)}"
-                                    f"_PLAYER{nplayer}"
-                                    f"_{util.safeFileName(subject)}"
-                                    f"_{util.safeFileName(behavior)}"
-                                    f'_{start:.3f}_%08d.{self.frame_bitmap_format.lower()}"'
-                                )
 
-                                logging.debug(f"ffmpeg command: {ffmpeg_command}")
+                            else:
+                                continue
 
-                                p = subprocess.Popen(
-                                    ffmpeg_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True
-                                )
-                                out, _ = p.communicate()
+                        ffmpeg_command = (
+                            f'"{self.ffmpeg_bin}" '
+                            f"-ss {start:.3f} "
+                            f'-i "{media_path}" '
+                            f"-vframes {vframes} "
+                            f'"{exportDir}{os.sep}'
+                            f"{util.safeFileName(obsId).replace(' ', '-')}"
+                            f"_PLAYER{nplayer}"
+                            f"_{util.safeFileName(subject).replace(' ', '-')}"
+                            f"_{util.safeFileName(behavior).replace(' ', '-')}"
+                            f'_{start:.3f}_%08d.{self.frame_bitmap_format.lower()}"'
+                        )
+
+                        logging.debug(f"ffmpeg command: {ffmpeg_command}")
+
+                        p = subprocess.Popen(ffmpeg_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+                        out, _ = p.communicate()
 
 
 def extract_events(self):
@@ -350,12 +336,17 @@ def extract_events(self):
         results.ptText.appendHtml(out)
         results.pbSave.setVisible(False)
         results.pbCancel.setVisible(True)
-        if not results.exec_():
+        if results.exec_():
+            # remove live observations
+            selected_observations = [x for x in selected_observations if x not in live_images_obs_list]
+            if not selected_observations:
+                return
+        else:
             return
 
-    # remove live observations
-    selected_observations = [x for x in selected_observations if x not in live_images_obs_list]
-    if not selected_observations:
+    # check if state events are paired
+    not_ok, selected_observations = project_functions.check_state_events(self.pj, selected_observations)
+    if not_ok or not selected_observations:
         return
 
     parameters = select_subj_behav.choose_obs_subj_behav_category(
@@ -366,13 +357,11 @@ def extract_events(self):
         flagShowIncludeModifiers=False,
         flagShowExcludeBehaviorsWoEvents=False,
     )
-
-    if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
+    if parameters == {}:
         return
 
-    # check if state events are paired
-    not_ok, selected_observations = project_functions.check_state_events(self.pj, selected_observations)
-    if not_ok or not selected_observations:
+    if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
+        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to analyze")
         return
 
     # Ask for time interval around the event
@@ -395,13 +384,13 @@ def extract_events(self):
     if not ok:
         return
 
-    exportDir = QFileDialog().getExistingDirectory(
+    export_dir = QFileDialog().getExistingDirectory(
         self,
         "Choose a directory to extract events",
         os.path.expanduser("~"),
         options=QFileDialog(self).ShowDirsOnly,
     )
-    if not exportDir:
+    if not export_dir:
         return
 
     cursor = db_functions.load_events_in_db(
@@ -412,22 +401,25 @@ def extract_events(self):
         time_interval=cfg.TIME_FULL_OBS,
     )
 
+    """
     ffmpeg_extract_command = (
         '"{ffmpeg_bin}" -i "{input_}" -y -ss {start} -to {stop} {codecs} '
         ' "{dir_}{sep}{obsId}_{player}_{subject}_{behavior}_{globalStart}'
         '-{globalStop}{extension}" '
     )
+    """
+    ffmpeg_extract_command = '"{ffmpeg_bin}" -ss {start} -i "{input_}" -y -t {duration} {codecs} '
+    mem_command = ""
+    for obs_id in selected_observations:
 
-    for obsId in selected_observations:
+        for nplayer in self.pj[cfg.OBSERVATIONS][obs_id][cfg.FILE]:
 
-        for nplayer in self.pj[cfg.OBSERVATIONS][obsId][cfg.FILE]:
-
-            if not self.pj[cfg.OBSERVATIONS][obsId][cfg.FILE][nplayer]:
+            if not self.pj[cfg.OBSERVATIONS][obs_id][cfg.FILE][nplayer]:
                 continue
 
             duration1 = []  # in seconds
-            for mediaFile in self.pj[cfg.OBSERVATIONS][obsId][cfg.FILE][nplayer]:
-                duration1.append(self.pj[cfg.OBSERVATIONS][obsId][cfg.MEDIA_INFO][cfg.LENGTH][mediaFile])
+            for mediaFile in self.pj[cfg.OBSERVATIONS][obs_id][cfg.FILE][nplayer]:
+                duration1.append(self.pj[cfg.OBSERVATIONS][obs_id][cfg.MEDIA_INFO][cfg.LENGTH][mediaFile])
 
             for subject in parameters[cfg.SELECTED_SUBJECTS]:
 
@@ -435,7 +427,7 @@ def extract_events(self):
 
                     cursor.execute(
                         "SELECT occurence FROM events WHERE observation = ? AND subject = ? AND code = ?",
-                        (obsId, subject, behavior),
+                        (obs_id, subject, behavior),
                     )
                     rows = [{"occurence": util.float2decimal(r["occurence"])} for r in cursor.fetchall()]
 
@@ -449,89 +441,90 @@ def extract_events(self):
                             idx1 for idx1, x in enumerate(duration1) if row["occurence"] >= sum(duration1[0:idx1])
                         ][-1]
 
-                        # check if media has video
-                        try:
-                            if self.pj[cfg.OBSERVATIONS][obsId][cfg.MEDIA_INFO][cfg.HAS_VIDEO][
-                                self.pj[cfg.OBSERVATIONS][obsId][cfg.FILE][nplayer][mediaFileIdx]
-                            ]:
-                                codecs = "-acodec copy -vcodec copy"
-                                # extract extension from video file
-                                extension = pathlib.Path(
-                                    self.pj[cfg.OBSERVATIONS][obsId][cfg.FILE][nplayer][mediaFileIdx]
-                                ).suffix
-                                if not extension:
-                                    extension = ".mp4"
+                        if "VIDEO" in items_to_extract.upper():
+                            # check if media has video
+                            has_video = False
+                            try:
+                                has_video = self.pj[cfg.OBSERVATIONS][obs_id][cfg.MEDIA_INFO][cfg.HAS_VIDEO][
+                                    self.pj[cfg.OBSERVATIONS][obs_id][cfg.FILE][nplayer][mediaFileIdx]
+                                ]
+                            except:
+                                has_video = False
+                            if not has_video:
+                                if (
+                                    dialog.MessageDialog(
+                                        cfg.programName,
+                                        f"The media file {self.pj[cfg.OBSERVATIONS][obs_id][cfg.FILE][nplayer][mediaFileIdx]} does not have a video stream",
+                                        ["Continue", "Abort"],
+                                    )
+                                    == "Abort"
+                                ):
+                                    return
+                                else:
+                                    continue
+
+                            new_extension = ".mp4"
+                            if items_to_extract == "Only video":
+                                codecs = "-an"
                             else:
-                                codecs = "-vn"
-                                extension = ".wav"
+                                codecs = ""
 
-                                logging.debug(
-                                    f"Media {self.pj[cfg.OBSERVATIONS][obsId][cfg.FILE][nplayer][mediaFileIdx]} does not have video"
-                                )
+                        if items_to_extract == "Only audio":
+                            # check if media has audio
+                            has_audio = False
+                            try:
+                                has_audio = self.pj[cfg.OBSERVATIONS][obs_id][cfg.MEDIA_INFO][cfg.HAS_AUDIO][
+                                    self.pj[cfg.OBSERVATIONS][obs_id][cfg.FILE][nplayer][mediaFileIdx]
+                                ]
+                            except:
+                                has_audio = False
+                            if not has_audio:
 
-                        except Exception:
+                                if (
+                                    dialog.MessageDialog(
+                                        cfg.programName,
+                                        f"The media file {self.pj[cfg.OBSERVATIONS][obs_id][cfg.FILE][nplayer][mediaFileIdx]} does not have an audio stream",
+                                        ["Continue", "Abort"],
+                                    )
+                                    == "Abort"
+                                ):
+                                    return
+                                else:
+                                    continue
 
-                            logging.debug(
-                                f"has_video not found for: {self.pj[cfg.OBSERVATIONS][obsId][cfg.FILE][nplayer][mediaFileIdx]}"
-                            )
-
-                            continue
-
-                        globalStart = (
-                            dec("0.000") if row["occurence"] < timeOffset else round(row["occurence"] - timeOffset, 3)
-                        )
-                        start = round(
-                            row["occurence"]
-                            - timeOffset
-                            - util.float2decimal(sum(duration1[0:mediaFileIdx]))
-                            - self.pj[cfg.OBSERVATIONS][obsId][cfg.TIME_OFFSET],
-                            3,
-                        )
-                        if start < timeOffset:
-                            start = dec("0.000")
+                            new_extension = ".wav"
+                            codecs = "-vn"
 
                         if cfg.POINT in behavior_state:
 
+                            globalStart = (
+                                dec("0.000")
+                                if row["occurence"] < timeOffset
+                                else round(row["occurence"] - timeOffset, 3)
+                            )
+                            start = round(
+                                row["occurence"]
+                                - (timeOffset if timeOffset else 1)  # if time offset is not set default = 1 s
+                                - util.float2decimal(sum(duration1[0:mediaFileIdx]))
+                                - self.pj[cfg.OBSERVATIONS][obs_id][cfg.TIME_OFFSET],
+                                3,
+                            )
+                            if start < timeOffset:
+                                start = dec("0.000")
+
                             globalStop = round(row["occurence"] + timeOffset, 3)
 
                             stop = round(
                                 row["occurence"]
-                                + timeOffset
+                                + (timeOffset if timeOffset else 1)  # if time offset is not set default = 1 s
                                 - util.float2decimal(sum(duration1[0:mediaFileIdx]))
-                                - self.pj[cfg.OBSERVATIONS][obsId][cfg.TIME_OFFSET],
+                                - self.pj[cfg.OBSERVATIONS][obs_id][cfg.TIME_OFFSET],
                                 3,
                             )
 
-                            ffmpeg_command = ffmpeg_extract_command.format(
-                                ffmpeg_bin=self.ffmpeg_bin,
-                                input_=project_functions.full_path(
-                                    self.pj[cfg.OBSERVATIONS][obsId][cfg.FILE][nplayer][mediaFileIdx],
-                                    self.projectFileName,
-                                ),
-                                start=start,
-                                stop=stop,
-                                codecs=codecs,
-                                globalStart=globalStart,
-                                globalStop=globalStop,
-                                dir_=exportDir,
-                                sep=os.sep,
-                                obsId=util.safeFileName(obsId),
-                                player="PLAYER{}".format(nplayer),
-                                subject=util.safeFileName(subject),
-                                behavior=util.safeFileName(behavior),
-                                extension=extension,
-                            )
-
-                            logging.debug(f"ffmpeg command: {ffmpeg_command}")
-
-                            p = subprocess.Popen(
-                                ffmpeg_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True
-                            )
-                            out, _ = p.communicate()
-
                         if cfg.STATE in behavior_state:
-                            if idx % 2 == 0:
 
+                            if idx % 2 == 0:
                                 # check if stop is on same media file
                                 if (
                                     mediaFileIdx
@@ -554,46 +547,77 @@ def extract_events(self):
                                     if response == "Abort":
                                         return
 
+                                globalStart = (
+                                    dec("0.000")
+                                    if row["occurence"] < timeOffset
+                                    else round(row["occurence"] - timeOffset, 3)
+                                )
+                                start = round(
+                                    row["occurence"]
+                                    - timeOffset
+                                    - util.float2decimal(sum(duration1[0:mediaFileIdx]))
+                                    - self.pj[cfg.OBSERVATIONS][obs_id][cfg.TIME_OFFSET],
+                                    3,
+                                )
+                                if start < timeOffset:
+                                    start = dec("0.000")
+
                                 globalStop = round(rows[idx + 1]["occurence"] + timeOffset, 3)
 
                                 stop = round(
                                     rows[idx + 1]["occurence"]
                                     + timeOffset
-                                    - util.float2decimal(sum(duration1[0:mediaFileIdx])),
+                                    - util.float2decimal(sum(duration1[0:mediaFileIdx]))
+                                    - self.pj[cfg.OBSERVATIONS][obs_id][cfg.TIME_OFFSET],
                                     3,
                                 )
 
                                 # check if start after length of media
                                 if (
                                     start
-                                    > self.pj[cfg.OBSERVATIONS][obsId][cfg.MEDIA_INFO][cfg.LENGTH][
-                                        self.pj[cfg.OBSERVATIONS][obsId][cfg.FILE][nplayer][mediaFileIdx]
+                                    > self.pj[cfg.OBSERVATIONS][obs_id][cfg.MEDIA_INFO][cfg.LENGTH][
+                                        self.pj[cfg.OBSERVATIONS][obs_id][cfg.FILE][nplayer][mediaFileIdx]
                                     ]
                                 ):
                                     continue
 
-                                ffmpeg_command = ffmpeg_extract_command.format(
-                                    ffmpeg_bin=self.ffmpeg_bin,
-                                    input_=project_functions.full_path(
-                                        self.pj[cfg.OBSERVATIONS][obsId][cfg.FILE][nplayer][mediaFileIdx],
-                                        self.projectFileName,
-                                    ),
-                                    start=start,
-                                    stop=stop,
-                                    codecs=codecs,
-                                    globalStart=globalStart,
-                                    globalStop=globalStop,
-                                    dir_=exportDir,
-                                    sep=os.sep,
-                                    obsId=util.safeFileName(obsId),
-                                    player=f"PLAYER{nplayer}",
-                                    subject=util.safeFileName(subject),
-                                    behavior=util.safeFileName(behavior),
-                                    extension=extension,
-                                )
+                            else:
+                                continue
 
-                                logging.debug("ffmpeg command: {}".format(ffmpeg_command))
-                                p = subprocess.Popen(
-                                    ffmpeg_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True
+                        new_file_name = pl.Path(export_dir) / pl.Path(
+                            f"{util.safeFileName(obs_id).replace(' ', '-')}_PLAYER{nplayer}_{util.safeFileName(subject).replace(' ', '-')}_{util.safeFileName(behavior)}_{globalStart}-{globalStop}"
+                        ).with_suffix(new_extension)
+                        if new_file_name.is_file():
+                            if mem_command not in (cfg.OVERWRITE_ALL, cfg.SKIP_ALL):
+                                mem_command = dialog.MessageDialog(
+                                    cfg.programName,
+                                    f"The file <b>{new_file_name}</b> already exists.",
+                                    [cfg.OVERWRITE, cfg.OVERWRITE_ALL, cfg.SKIP, cfg.SKIP_ALL, cfg.CANCEL],
                                 )
-                                out, error = p.communicate()
+                            if mem_command == cfg.CANCEL:
+                                return
+                            if "SKIP" in mem_command.upper():
+                                continue
+
+                        ffmpeg_command = ffmpeg_extract_command.format(
+                            ffmpeg_bin=self.ffmpeg_bin,
+                            input_=project_functions.full_path(
+                                self.pj[cfg.OBSERVATIONS][obs_id][cfg.FILE][nplayer][mediaFileIdx],
+                                self.projectFileName,
+                            ),
+                            start=start,
+                            duration=stop - start,
+                            codecs=codecs,
+                        )
+
+                        logging.debug(f"ffmpeg command: {ffmpeg_command} '{new_file_name}'")
+
+                        p = subprocess.Popen(
+                            f"{ffmpeg_command} '{new_file_name}'",
+                            stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE,
+                            shell=True,
+                        )
+                        out, _ = p.communicate()
+
+    self.statusbar.showMessage(f"Media sequence extracted in {export_dir}", 0)
