@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2022 Olivier Friard
+Copyright 2012-2023 Olivier Friard
 
 This file is part of BORIS.
 
@@ -22,8 +22,8 @@ This file is part of BORIS.
 
 import datetime as dt
 import logging
+import math
 import os
-import sys
 import tablib
 import pathlib as pl
 from decimal import Decimal as dec
@@ -38,7 +38,7 @@ from . import project_functions
 from . import dialog
 from . import db_functions
 
-from PyQt5.QtWidgets import QFileDialog, QMessageBox, QInputDialog
+from PyQt5.QtWidgets import QApplication, QFileDialog, QMessageBox, QInputDialog
 
 
 def export_events_as_behavioral_sequences(self, separated_subjects=False, timed=False):
@@ -91,7 +91,10 @@ def export_events_as_behavioral_sequences(self, separated_subjects=False, timed=
         n_observations=len(selected_observations),
     )
 
+    if parameters == {}:
+        return
     if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
+        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to analyze")
         return
 
     fn = QFileDialog().getSaveFileName(
@@ -182,8 +185,10 @@ def export_tabular_events(self, mode: str = "tabular") -> None:
         flagShowExcludeBehaviorsWoEvents=False,
         n_observations=len(selected_observations),
     )
-
+    if parameters == {}:
+        return
     if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
+        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to analyze")
         return
 
     if mode == "tabular":
@@ -303,8 +308,9 @@ def export_aggregated_events(self):
             "Description": str,
             "Observation type": str,
             "Source": str,
-            "Total length": float,
-            "FPS": float,
+            "Total duration": float,
+            "Media duration (s)": float,
+            "FPS (frame/s)": float,
         }
         if cfg.INDEPENDENT_VARIABLES in self.pj:
             for idx in util.sorted_keys(self.pj[cfg.INDEPENDENT_VARIABLES]):
@@ -344,6 +350,7 @@ def export_aggregated_events(self):
                 "Start (s)": float,
                 "Stop (s)": float,
                 "Duration (s)": float,
+                "Media file name": str,
                 "Image index start": float,  # add image index and image file path to header
                 "Image index stop": float,
                 "Image file path start": str,
@@ -391,8 +398,10 @@ def export_aggregated_events(self):
         flagShowExcludeBehaviorsWoEvents=False,
         n_observations=len(selected_observations),
     )
-
+    if parameters == {}:
+        return
     if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
+        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to analyze")
         return
 
     # check for grouping results
@@ -673,7 +682,406 @@ def export_aggregated_events(self):
             QMessageBox.warning(None, cfg.programName, msg, QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
 
 
-def export_events_as_textgrid(self):
+def export_events_as_textgrid(self) -> None:
+    """
+    * select observations
+    * select subjects, behaviors and time interval
+    * export state events of selected observations as Praat textgrid
+    """
+
+    _, selected_observations = select_observations.select_observations(self.pj, mode=cfg.MULTIPLE, windows_title="")
+
+    if not selected_observations:
+        return
+
+    # check if coded behaviors are defined in ethogram
+    if project_functions.check_coded_behaviors_in_obs_list(self.pj, selected_observations):
+        return
+
+    # check if state events are paired
+    not_ok, selected_observations = project_functions.check_state_events(self.pj, selected_observations)
+    if not_ok or not selected_observations:
+        return
+
+    max_obs_length, _ = observation_operations.observation_length(self.pj, selected_observations)
+
+    # exit with message if events do not have timestamp
+    if max_obs_length.is_nan():
+        QMessageBox.critical(
+            None,
+            cfg.programName,
+            ("This function is not available for observations with events that do not have timestamp"),
+            QMessageBox.Ok | QMessageBox.Default,
+            QMessageBox.NoButton,
+        )
+        return
+
+    start_coding, end_coding, _ = observation_operations.coding_time(self.pj[cfg.OBSERVATIONS], selected_observations)
+
+    parameters = select_subj_behav.choose_obs_subj_behav_category(
+        self,
+        selected_observations,
+        start_coding=start_coding,
+        end_coding=end_coding,
+        flagShowIncludeModifiers=False,
+        flagShowExcludeBehaviorsWoEvents=False,
+        maxTime=max_obs_length,
+        n_observations=len(selected_observations),
+    )
+    if parameters == {}:
+        return
+    if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
+        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to export")
+        return
+
+    export_dir = QFileDialog(self).getExistingDirectory(
+        self, "Export events as Praat TextGrid", os.path.expanduser("~"), options=QFileDialog(self).ShowDirsOnly
+    )
+    if not export_dir:
+        return
+
+    mem_command = ""
+
+    # see https://www.fon.hum.uva.nl/praat/manual/TextGrid_file_formats.html
+
+    interval_subject_header = (
+        "    item [{subject_index}]:\n"
+        '        class = "IntervalTier"\n'
+        '        name = "{subject}"\n'
+        "        xmin = 0.0\n"
+        "        xmax = {intervalsMax}\n"
+        "        intervals: size = {intervalsSize}\n"
+    )
+
+    interval_template = (
+        "        intervals [{count}]:\n"
+        "            xmin = {xmin}\n"
+        "            xmax = {xmax}\n"
+        '            text = "{name}"\n'
+    )
+
+    point_subject_header = (
+        "    item [{subject_index}]:\n"
+        '        class = "TextTier"\n'
+        '        name = "{subject}"\n'
+        "        xmin = {intervalsMin}\n"
+        "        xmax = {intervalsMax}\n"
+        "        points: size = {intervalsSize}\n"
+    )
+
+    point_template = "        points [{count}]:\n" "            number = {number}\n" '            mark = "{mark}"\n'
+
+    # widget for results
+    self.results = dialog.Results_dialog()
+    self.results.setWindowTitle(f"{cfg.programName} - Export events as Praat TextGrid")
+    self.results.show()
+
+    ok, msg, db_connector = db_functions.load_aggregated_events_in_db(
+        self.pj, parameters[cfg.SELECTED_SUBJECTS], selected_observations, parameters[cfg.SELECTED_BEHAVIORS]
+    )
+
+    if db_connector is None:
+        logging.critical(f"Error when loading aggregated events in DB")
+        return
+
+    cursor = db_connector.cursor()
+
+    file_count: int = 0
+
+    for obs_id in selected_observations:
+
+        if parameters["time"] == cfg.TIME_EVENTS:
+            start_coding, end_coding, coding_duration = observation_operations.coding_time(
+                self.pj[cfg.OBSERVATIONS], [obs_id]
+            )
+            if start_coding is None and end_coding is None:  # no events
+                self.results.ptText.appendHtml(f"The observation <b>{obs_id}</b> does not have events.")
+                QApplication.processEvents()
+                continue
+
+            if math.isnan(start_coding) or math.isnan(end_coding):  # obs with no timestamp
+                self.results.ptText.appendHtml(f"The observation <b>{obs_id}</b> does not have timestamp.")
+                QApplication.processEvents()
+                continue
+
+            min_time = float(start_coding)
+            max_time = float(end_coding)
+
+        if parameters["time"] == cfg.TIME_FULL_OBS:
+
+            if self.pj[cfg.OBSERVATIONS][obs_id][cfg.TYPE] == cfg.MEDIA:
+                max_media_duration, _ = observation_operations.media_duration(self.pj[cfg.OBSERVATIONS], [obs_id])
+                min_time = float(0)
+                max_time = float(max_media_duration)
+                coding_duration = max_media_duration
+
+            if self.pj[cfg.OBSERVATIONS][obs_id][cfg.TYPE] in (cfg.LIVE, cfg.IMAGES):
+                start_coding, end_coding, coding_duration = observation_operations.coding_time(
+                    self.pj[cfg.OBSERVATIONS], [obs_id]
+                )
+                if start_coding is None and end_coding is None:  # no events
+                    self.results.ptText.appendHtml(f"The observation <b>{obs_id}</b> does not have events.")
+                    QApplication.processEvents()
+                    continue
+                if math.isnan(start_coding) or math.isnan(end_coding):  # obs with no timestamp
+                    self.results.ptText.appendHtml(f"The observation <b>{obs_id}</b> does not have timestamp.")
+                    QApplication.processEvents()
+                    continue
+
+                min_time = float(start_coding)
+                max_time = float(end_coding)
+
+        if parameters["time"] == cfg.TIME_ARBITRARY_INTERVAL:
+            min_time = float(parameters[cfg.START_TIME])
+            max_time = float(parameters[cfg.END_TIME])
+
+        """
+        print(f"min_time: {min_time}")
+        print(f"max_time: {max_time}")
+        """
+
+        # delete events outside time interval
+        cursor.execute(
+            "DELETE FROM aggregated_events WHERE observation = ? AND (start < ? AND stop < ?) OR (start > ? AND stop > ?)",
+            (
+                obs_id,
+                min_time,
+                min_time,
+                max_time,
+                max_time,
+            ),
+        )
+        cursor.execute(
+            "UPDATE aggregated_events SET start = ? WHERE observation = ? AND start < ? AND stop BETWEEN ? AND ?",
+            (
+                min_time,
+                obs_id,
+                min_time,
+                min_time,
+                max_time,
+            ),
+        )
+        cursor.execute(
+            "UPDATE aggregated_events SET stop = ? WHERE observation = ? AND stop > ? AND start BETWEEN ? AND ?",
+            (
+                max_time,
+                obs_id,
+                max_time,
+                min_time,
+                max_time,
+            ),
+        )
+        cursor.execute(
+            "UPDATE aggregated_events SET start = ?, stop = ? WHERE observation = ? AND start < ? AND stop > ?",
+            (
+                min_time,
+                max_time,
+                obs_id,
+                min_time,
+                max_time,
+            ),
+        )
+
+        next_obs: bool = False
+
+        """
+        total_media_duration = round(
+            observation_operations.observation_total_length(self.pj[cfg.OBSERVATIONS][obs_id]), 3
+        )
+        """
+
+        cursor.execute(
+            (
+                "SELECT COUNT(DISTINCT subject) FROM aggregated_events "
+                "WHERE observation = ? AND subject IN ({}) AND type = 'STATE' ".format(
+                    ",".join(["?"] * len(parameters[cfg.SELECTED_SUBJECTS]))
+                )
+            ),
+            [obs_id] + parameters[cfg.SELECTED_SUBJECTS],
+        )
+
+        subjectsNum = int(cursor.fetchone()[0])
+        subjectsMin, subjectsMax = min_time, max_time
+
+        out = (
+            'File type = "ooTextFile"\n'
+            'Object class = "TextGrid"\n'
+            "\n"
+            f"xmin = 0.0\n"
+            f"xmax = {subjectsMax}\n"
+            "tiers? <exists>\n"
+            f"size = {subjectsNum}\n"
+            "item []:\n"
+        )
+
+        subject_index = 0
+        for subject in parameters[cfg.SELECTED_SUBJECTS]:
+            if subject not in [
+                x[cfg.EVENT_SUBJECT_FIELD_IDX] if x[cfg.EVENT_SUBJECT_FIELD_IDX] else cfg.NO_FOCAL_SUBJECT
+                for x in self.pj[cfg.OBSERVATIONS][obs_id][cfg.EVENTS]
+            ]:
+                continue
+
+            intervalsMin, intervalsMax = min_time, max_time
+
+            # STATE events
+            cursor.execute(
+                (
+                    "SELECT start, stop, behavior FROM aggregated_events "
+                    "WHERE observation = ? AND subject = ? AND type = 'STATE' ORDER BY start"
+                ),
+                (obs_id, subject),
+            )
+
+            rows = [
+                {"start": util.float2decimal(r["start"]), "stop": util.float2decimal(r["stop"]), "code": r["behavior"]}
+                for r in cursor.fetchall()
+            ]
+            if not rows:
+                continue
+
+            out += interval_subject_header
+
+            count = 0
+
+            # check if 1st behavior starts at the beginning
+            if rows[0]["start"] > 0:
+                count += 1
+                out += interval_template.format(count=count, name="null", xmin=0.0, xmax=rows[0]["start"])
+
+            for idx, row in enumerate(rows):
+
+                # check if events are overlapping
+                if (idx + 1 < len(rows)) and (row["stop"] > rows[idx + 1]["start"]):
+
+                    self.results.ptText.appendHtml(
+                        (
+                            f"The events overlap for subject <b>{subject}</b> in the observation <b>{obs_id}</b>. "
+                            "It is not possible to create the Praat TextGrid file."
+                        )
+                    )
+                    QApplication.processEvents()
+
+                    next_obs = True
+                    break
+
+                count += 1
+
+                """
+                print(f'{row["start"]=}   {row["stop"]=}')
+                if idx + 1 < len(rows):
+                    print(f'{rows[idx + 1]["start"]=}')
+                print()
+                """
+
+                if (idx + 1 < len(rows)) and (
+                    rows[idx + 1]["start"] - dec("0.001") <= row["stop"] < rows[idx + 1]["start"]
+                ):
+                    xmax = rows[idx + 1]["start"]
+                else:
+                    xmax = row["stop"]
+
+                out += interval_template.format(count=count, name=row["code"], xmin=row["start"], xmax=xmax)
+
+                # check if no behavior
+                if (idx + 1 < len(rows)) and (row["stop"] < rows[idx + 1]["start"] - dec("0.001")):
+                    count += 1
+                    out += interval_template.format(
+                        count=count,
+                        name="null",
+                        xmin=row["stop"],
+                        xmax=rows[idx + 1]["start"],
+                    )
+
+            if next_obs:
+                break
+
+            # check if last event ends at the end of media file
+            if rows[-1]["stop"] < max_time:
+                count += 1
+                out += interval_template.format(count=count, name="null", xmin=rows[-1]["stop"], xmax=max_time)
+
+            # add info
+            subject_index += 1
+            out = out.format(
+                subject_index=subject_index,
+                subject=subject,
+                intervalsSize=count,
+                intervalsMin=intervalsMin,
+                intervalsMax=intervalsMax,
+            )
+
+            # POINT events
+            cursor.execute(
+                (
+                    "SELECT start, behavior FROM aggregated_events "
+                    "WHERE observation = ? AND subject = ? AND type = 'POINT' ORDER BY start"
+                ),
+                (obs_id, subject),
+            )
+
+            rows = [{"start": util.float2decimal(r["start"]), "code": r["behavior"]} for r in cursor.fetchall()]
+            if not rows:
+                continue
+
+            out += point_subject_header
+
+            count = 0
+
+            for idx, row in enumerate(rows):
+
+                count += 1
+                out += point_template.format(count=count, mark=row["code"], number=row["start"])
+
+            # add info
+            subject_index += 1
+            out = out.format(
+                subject_index=subject_index,
+                subject=subject,
+                intervalsSize=count,
+                intervalsMin=intervalsMin,
+                intervalsMax=intervalsMax,
+            )
+
+        if next_obs:
+            continue
+
+        # check if file already exists
+        if (
+            mem_command != cfg.OVERWRITE_ALL
+            and pl.Path(f"{pl.Path(export_dir) / util.safeFileName(obs_id)}.textGrid").is_file()
+        ):
+            if mem_command == cfg.SKIP_ALL:
+                continue
+            mem_command = dialog.MessageDialog(
+                cfg.programName,
+                f"The file <b>{pl.Path(export_dir) / util.safeFileName(obs_id)}.textGrid</b> already exists.",
+                [cfg.OVERWRITE, cfg.OVERWRITE_ALL, cfg.SKIP, cfg.SKIP_ALL, cfg.CANCEL],
+            )
+            if mem_command == cfg.CANCEL:
+                return
+            if mem_command in (cfg.SKIP, cfg.SKIP_ALL):
+                continue
+
+        try:
+            with open(f"{pl.Path(export_dir) / util.safeFileName(obs_id)}.textGrid", "w") as f:
+                f.write(out)
+            file_count += 1
+            self.results.ptText.appendHtml(
+                f"File {pl.Path(export_dir) / util.safeFileName(obs_id)}.textGrid was created."
+            )
+            QApplication.processEvents()
+        except Exception:
+            self.results.ptText.appendHtml(
+                f"The file {pl.Path(export_dir) / util.safeFileName(obs_id)}.textGrid can not be created."
+            )
+            QApplication.processEvents()
+
+    self.results.ptText.appendHtml(f"Done.  {file_count} file(s) were created in {export_dir}.")
+    QApplication.processEvents()
+
+
+def export_events_as_textgrid_old(self):
     """
     export state events as Praat textgrid
     """
@@ -716,8 +1124,10 @@ def export_events_as_textgrid(self):
         flagShowExcludeBehaviorsWoEvents=False,
         n_observations=len(selected_observations),
     )
-
+    if parameters == {}:
+        return
     if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
+        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to export")
         return
 
     exportDir = QFileDialog(self).getExistingDirectory(
@@ -757,20 +1167,28 @@ def export_events_as_textgrid(self):
 
     point_template = "        points [{count}]:\n" "            number = {number}\n" '            mark = "{mark}"\n'
 
+    # widget for results
+    self.results = dialog.Results_dialog()
+    self.results.setWindowTitle(f"{cfg.programName} - Export events as Praat TextGrid")
+    self.results.show()
+
+    cursor = db_functions.load_events_in_db(
+        self.pj,
+        parameters[cfg.SELECTED_SUBJECTS],
+        selected_observations,
+        parameters[cfg.SELECTED_BEHAVIORS],
+    )
+
+    file_count: int = 0
+
     for obs_id in selected_observations:
 
+        next_obs: bool = False
+
         total_media_duration = round(
             observation_operations.observation_total_length(self.pj[cfg.OBSERVATIONS][obs_id]), 3
         )
 
-        cursor = db_functions.load_events_in_db(
-            self.pj,
-            parameters[cfg.SELECTED_SUBJECTS],
-            selected_observations,
-            parameters[cfg.SELECTED_BEHAVIORS],
-            time_interval=cfg.TIME_FULL_OBS,
-        )
-
         cursor.execute(
             (
                 "SELECT COUNT(DISTINCT subject) FROM events "
@@ -798,7 +1216,10 @@ def export_events_as_textgrid(self):
 
         subject_index = 0
         for subject in parameters[cfg.SELECTED_SUBJECTS]:
-            if subject not in [x[cfg.EVENT_SUBJECT_FIELD_IDX] for x in self.pj[cfg.OBSERVATIONS][obs_id][cfg.EVENTS]]:
+            if subject not in [
+                x[cfg.EVENT_SUBJECT_FIELD_IDX] if x[cfg.EVENT_SUBJECT_FIELD_IDX] else cfg.NO_FOCAL_SUBJECT
+                for x in self.pj[cfg.OBSERVATIONS][obs_id][cfg.EVENTS]
+            ]:
                 continue
 
             intervalsMin, intervalsMax = 0, total_media_duration
@@ -827,18 +1248,21 @@ def export_events_as_textgrid(self):
                 out += interval_template.format(count=count, name="null", xmin=0.0, xmax=rows[0]["occurence"])
 
             for idx, row in enumerate(rows):
-                if idx % 2 == 0:
 
+                if idx % 2 == 0:
                     # check if events not interlacced
                     if row["code"] != rows[idx + 1]["code"]:
-                        QMessageBox.critical(
-                            None,
-                            cfg.programName,
-                            "The events are interlaced. It is not possible to produce the Praat TextGrid file",
-                            QMessageBox.Ok | QMessageBox.Default,
-                            QMessageBox.NoButton,
+
+                        self.results.ptText.appendHtml(
+                            (
+                                f"The events are interlaced for subject <b>{subject}</b> in the observation <b>{obs_id}</b>. "
+                                "It is not possible to create the Praat TextGrid file."
+                            )
                         )
-                        return
+                        QApplication.processEvents()
+
+                        next_obs = True
+                        break
 
                     count += 1
                     out += interval_template.format(
@@ -859,6 +1283,9 @@ def export_events_as_textgrid(self):
                         else:
                             rows[idx + 2]["occurence"] = rows[idx + 1]["occurence"]
 
+            if next_obs:
+                break
+
             # check if last event ends at the end of media file
             if rows[-1]["occurence"] < observation_operations.observation_total_length(
                 self.pj[cfg.OBSERVATIONS][obs_id]
@@ -910,6 +1337,9 @@ def export_events_as_textgrid(self):
                 intervalsMax=intervalsMax,
             )
 
+        if next_obs:
+            continue
+
         # check if file already exists
         if (
             mem_command != cfg.OVERWRITE_ALL
@@ -924,12 +1354,22 @@ def export_events_as_textgrid(self):
             )
             if mem_command == cfg.CANCEL:
                 return
-            if mem_command in [cfg.SKIP, cfg.SKIP_ALL]:
+            if mem_command in (cfg.SKIP, cfg.SKIP_ALL):
                 continue
 
         try:
             with open(f"{pl.Path(exportDir) / util.safeFileName(obs_id)}.textGrid", "w") as f:
                 f.write(out)
-
+            file_count += 1
+            self.results.ptText.appendHtml(
+                f"File {pl.Path(exportDir) / util.safeFileName(obs_id)}.textGrid was created."
+            )
+            QApplication.processEvents()
         except Exception:
-            QMessageBox.critical(self, cfg.programName, "The file can not be saved")
+            self.results.ptText.appendHtml(
+                f"The file {pl.Path(exportDir) / util.safeFileName(obs_id)}.textGrid can not be created."
+            )
+            QApplication.processEvents()
+
+    self.results.ptText.appendHtml(f"Done.  {file_count} file(s) were created in {exportDir}.")
+    QApplication.processEvents()
