@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2022 Olivier Friard
+Copyright 2012-2023 Olivier Friard
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -374,6 +374,7 @@ def export_tabular_events(
         "Observation duration": float,
         "Observation type": str,
         "Source": str,
+        "Media duration (s)": str,
         "FPS": float,
     }
 
@@ -424,16 +425,35 @@ def export_tabular_events(
             fields: list = []
             fields.append(obsId)
             fields.append(observation.get("date", "").replace("T", " "))
-            fields.append(util.eol2space(observation.get("description", "")))
+            fields.append(util.eol2space(observation.get(cfg.DESCRIPTION, "")))
             # total length
             fields.append(coding_duration if not coding_duration.is_nan() else cfg.NA)
 
             if observation[cfg.TYPE] == cfg.MEDIA:
                 fields.append("Media file(s)")
 
-                media_file_str, fps_str = "", ""
+                media_file_str, fps_str, media_durations_str = "", "", ""
+                for player in observation[cfg.FILE]:
+                    media_file_lst, fps_lst, media_durations_lst = [], [], []
+                    if observation[cfg.FILE][player]:
+                        for media_file in observation[cfg.FILE][player]:
+                            media_file_lst.append(media_file)
+                            fps_lst.append(f"{observation[cfg.MEDIA_INFO][cfg.FPS].get(media_file, cfg.NA):.3f}")
+                            media_durations_lst.append(
+                                f"{observation[cfg.MEDIA_INFO][cfg.LENGTH].get(media_file, cfg.NA):.3f}"
+                            )
+                        if player > "1":
+                            media_file_str += "|"
+                            fps_str += "|"
+                            media_durations_str += "|"
+                        media_file_str += f"player #{player}:" + ";".join(media_file_lst)
+                        fps_str += ";".join(fps_lst)
+                        media_durations_str += ";".join(media_durations_lst)
+
+                """
                 # number of players
                 n_players = len([x for x in observation[cfg.FILE] if observation[cfg.FILE][x]])
+                media_file_str, fps_str = "", ""
                 for player in observation[cfg.FILE]:
                     if observation[cfg.FILE][player]:
                         if media_file_str:
@@ -449,12 +469,14 @@ def export_tabular_events(
                             fps_list.append(f"{observation[cfg.MEDIA_INFO][cfg.FPS].get(media_file, cfg.NA):.3f}")
                         media_file_str += ";".join(media_list)
                         fps_str += ";".join(fps_list)
+                """
 
                 fields.append(media_file_str)
 
             elif observation[cfg.TYPE] == cfg.LIVE:
                 fields.append("Live observation")
                 fields.append(cfg.NA)
+                media_durations_str = cfg.NA
                 fps_str = cfg.NA
 
             elif observation[cfg.TYPE] == cfg.IMAGES:
@@ -463,11 +485,15 @@ def export_tabular_events(
                 for dir in observation[cfg.DIRECTORIES_LIST]:
                     dir_list.append(dir)
                 fields.append(";".join(dir_list))
+                media_durations_str = cfg.NA
                 fps_str = cfg.NA
 
             else:
                 fields.append("")
 
+            # media duration
+            fields.append(media_durations_str)
+
             # FPS
             fields.append(fps_str)
 
@@ -504,21 +530,10 @@ def export_tabular_events(
 
             # check video file name containing the event
             if observation[cfg.TYPE] == cfg.MEDIA:
-                cumul_media_durations = [0]
-                for media_file in observation[cfg.FILE]["1"]:
-                    media_duration = observation[cfg.MEDIA_INFO][cfg.LENGTH][media_file]
-                    cumul_media_durations.append(cumul_media_durations[-1] + media_duration)
-
-                player_idx_list = [
-                    idx
-                    for idx, x in enumerate(cumul_media_durations)
-                    if cumul_media_durations[idx - 1] < event[cfg.EVENT_TIME_FIELD_IDX] <= x
-                ]
-                if len(player_idx_list):
-                    player_idx = player_idx_list[0] - 1
-                    video_file_name = observation[cfg.FILE]["1"][player_idx]
-                else:
-                    player_idx = -1
+                video_file_name = observation_operations.event2media_file_name(
+                    observation, event[cfg.EVENT_TIME_FIELD_IDX]
+                )
+                if video_file_name is None:
                     video_file_name = "Not found"
 
             elif observation[cfg.TYPE] in (cfg.LIVE, cfg.IMAGES):
@@ -1158,7 +1173,7 @@ def export_aggregated_events(pj: dict, parameters: dict, obsId: str) -> Tuple[ta
     logging.debug(f"min_time: {min_time}  max_time: {max_time}")
 
     # obs description
-    obs_description = observation[cfg.DESCRIPTION]
+    obs_description = util.eol2space(observation[cfg.DESCRIPTION])
 
     """
     obs_length = observation_operations.observation_total_length(pj[cfg.OBSERVATIONS][obsId])
@@ -1176,7 +1191,7 @@ def export_aggregated_events(pj: dict, parameters: dict, obsId: str) -> Tuple[ta
 
     # adapt start and stop to the selected time interval
     if not math.isnan(min_time) and not math.isnan(max_time):  # obs with timestamp
-
+        # delete events outside time interval
         cursor.execute(
             "DELETE FROM aggregated_events WHERE observation = ? AND (start < ? AND stop < ?) OR (start > ? AND stop > ?)",
             (
@@ -1262,22 +1277,41 @@ def export_aggregated_events(pj: dict, parameters: dict, obsId: str) -> Tuple[ta
 
                 for row in cursor.fetchall():
 
+                    media_file_name = cfg.NA
+
                     if observation[cfg.TYPE] == cfg.MEDIA:
                         observation_type = "Media file"
-                        media_file_str = ""
-                        fps_str = ""
+
+                        media_file_name = observation_operations.event2media_file_name(observation, row["start"])
+                        if media_file_name is None:
+                            media_file_name = "Not found"
+
+                        media_file_str, fps_str, media_durations_str = "", "", ""
+
                         for player in observation[cfg.FILE]:
+                            media_file_lst, fps_lst, media_durations_lst = [], [], []
                             if observation[cfg.FILE][player]:
-                                media_file_str += f"player #{player}: "
-                                fps_str += f"player #{player}: "
                                 for media_file in observation[cfg.FILE][player]:
-                                    media_file_str += f"{media_file}; "
-                                    fps_str += f"{observation[cfg.MEDIA_INFO][cfg.FPS].get(media_file, cfg.NA):.3f}; "
+                                    media_file_lst.append(media_file)
+                                    fps_lst.append(
+                                        f"{observation[cfg.MEDIA_INFO][cfg.FPS].get(media_file, cfg.NA):.3f}"
+                                    )
+                                    media_durations_lst.append(
+                                        f"{observation[cfg.MEDIA_INFO][cfg.LENGTH].get(media_file, cfg.NA):.3f}"
+                                    )
+                                if player > "1":
+                                    media_file_str += "|"
+                                    fps_str += "|"
+                                    media_durations_str += "|"
+                                media_file_str += f"player #{player}:" + ";".join(media_file_lst)
+                                fps_str += ";".join(fps_lst)
+                                media_durations_str += ";".join(media_durations_lst)
 
                     if observation[cfg.TYPE] == cfg.LIVE:
                         observation_type = "Live observation"
                         media_file_str = cfg.NA
                         fps_str = cfg.NA
+                        media_durations_str = cfg.NA
 
                     if observation[cfg.TYPE] == cfg.IMAGES:
                         observation_type = "From pictures"
@@ -1285,6 +1319,8 @@ def export_aggregated_events(pj: dict, parameters: dict, obsId: str) -> Tuple[ta
                         for dir in observation[cfg.DIRECTORIES_LIST]:
                             media_file_str += f"{dir}; "
                         fps_str = cfg.NA
+                        # TODO: number of pictures in each directory
+                        media_durations_str = cfg.NA
 
                     row_data = []
 
@@ -1296,6 +1332,7 @@ def export_aggregated_events(pj: dict, parameters: dict, obsId: str) -> Tuple[ta
                             observation_type,
                             media_file_str,
                             f"{coding_duration:.3f}" if not coding_duration.is_nan() else cfg.NA,
+                            media_durations_str,
                             fps_str,
                         ]
                     )
@@ -1341,6 +1378,7 @@ def export_aggregated_events(pj: dict, parameters: dict, obsId: str) -> Tuple[ta
                                 f"{row['start']:.3f}" if row["start"] is not None else cfg.NA,  # start
                                 f"{row['stop']:.3f}" if row["stop"] is not None else cfg.NA,  # stop
                                 cfg.NA,  # duration
+                                media_file_name,  # Media file name
                             ]
                         )
 
@@ -1354,6 +1392,7 @@ def export_aggregated_events(pj: dict, parameters: dict, obsId: str) -> Tuple[ta
                                 f"{row['stop'] - row['start']:.3f}"
                                 if (row["stop"] is not None) and (row["start"] is not None)
                                 else cfg.NA,
+                                media_file_name,  # Media file name
                             ]
                         )
 
