@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2022 Olivier Friard
+Copyright 2012-2023 Olivier Friard
 
 
   This program is free software; you can redistribute it and/or modify
@@ -293,7 +293,7 @@ def irr_cohen_kappa(self):
         )
         return
 
-    plot_parameters = select_subj_behav.choose_obs_subj_behav_category(
+    parameters = select_subj_behav.choose_obs_subj_behav_category(
         self,
         selected_observations,
         start_coding=dec("NaN"),
@@ -302,8 +302,10 @@ def irr_cohen_kappa(self):
         flagShowExcludeBehaviorsWoEvents=False,
         n_observations=len(selected_observations),
     )
-
-    if not plot_parameters[cfg.SELECTED_SUBJECTS] or not plot_parameters[cfg.SELECTED_BEHAVIORS]:
+    if parameters == {}:
+        return
+    if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
+        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to analyze")
         return
 
     # ask for time slice
@@ -315,14 +317,14 @@ def irr_cohen_kappa(self):
     interval = util.float2decimal(i)
 
     ok, msg, db_connector = db_functions.load_aggregated_events_in_db(
-        self.pj, plot_parameters[cfg.SELECTED_SUBJECTS], selected_observations, plot_parameters[cfg.SELECTED_BEHAVIORS]
+        self.pj, parameters[cfg.SELECTED_SUBJECTS], selected_observations, parameters[cfg.SELECTED_BEHAVIORS]
     )
 
     cursor = db_connector.cursor()
     out = (
         "Index of Inter-rater Reliability - Cohen's Kappa\n\n"
         f"Interval time: {interval:.3f} s\n"
-        f"Selected subjects: {', '.join(plot_parameters[cfg.SELECTED_SUBJECTS])}\n\n"
+        f"Selected subjects: {', '.join(parameters[cfg.SELECTED_SUBJECTS])}\n\n"
     )
 
     mem_done = []
@@ -338,8 +340,8 @@ def irr_cohen_kappa(self):
                     obs_id1,
                     obs_id2,
                     interval,
-                    plot_parameters[cfg.SELECTED_SUBJECTS],
-                    plot_parameters[cfg.INCLUDE_MODIFIERS],
+                    parameters[cfg.SELECTED_SUBJECTS],
+                    parameters[cfg.INCLUDE_MODIFIERS],
                 )
                 irr_results[selected_observations.index(obs_id1), selected_observations.index(obs_id2)] = K
                 irr_results[selected_observations.index(obs_id2), selected_observations.index(obs_id1)] = K
@@ -599,7 +601,7 @@ def needleman_wunch(self):
         )
         return
 
-    plot_parameters = select_subj_behav.choose_obs_subj_behav_category(
+    parameters = select_subj_behav.choose_obs_subj_behav_category(
         self,
         selected_observations,
         start_coding=dec("NaN"),
@@ -609,7 +611,11 @@ def needleman_wunch(self):
         n_observations=len(selected_observations),
     )
 
-    if not plot_parameters[cfg.SELECTED_SUBJECTS] or not plot_parameters[cfg.SELECTED_BEHAVIORS]:
+    if parameters == {}:
+        return
+
+    if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
+        QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to analyze")
         return
 
     # ask for time slice
@@ -620,14 +626,14 @@ def needleman_wunch(self):
     interval = util.float2decimal(i)
 
     ok, msg, db_connector = db_functions.load_aggregated_events_in_db(
-        self.pj, plot_parameters[cfg.SELECTED_SUBJECTS], selected_observations, plot_parameters[cfg.SELECTED_BEHAVIORS]
+        self.pj, parameters[cfg.SELECTED_SUBJECTS], selected_observations, parameters[cfg.SELECTED_BEHAVIORS]
     )
 
     cursor = db_connector.cursor()
     out = (
         "Needleman-Wunsch similarity\n\n"
         f"Time unit: {interval:.3f} s\n"
-        f"Selected subjects: {', '.join(plot_parameters[cfg.SELECTED_SUBJECTS])}\n\n"
+        f"Selected subjects: {', '.join(parameters[cfg.SELECTED_SUBJECTS])}\n\n"
     )
     mem_done = []
     nws_results = np.ones((len(selected_observations), len(selected_observations)))
@@ -642,8 +648,8 @@ def needleman_wunch(self):
                     obs_id1,
                     obs_id2,
                     interval,
-                    plot_parameters[cfg.SELECTED_SUBJECTS],
-                    plot_parameters[cfg.INCLUDE_MODIFIERS],
+                    parameters[cfg.SELECTED_SUBJECTS],
+                    parameters[cfg.INCLUDE_MODIFIERS],
                 )
                 nws_results[selected_observations.index(obs_id1), selected_observations.index(obs_id2)] = similarity
                 nws_results[selected_observations.index(obs_id2), selected_observations.index(obs_id1)] = similarity
