@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2022 Olivier Friard
+Copyright 2012-2023 Olivier Friard
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -57,6 +57,7 @@ from . import utilities as util
 from . import plot_data_module
 from . import player_dock_widget
 from . import gui_utilities
+from . import video_operations
 
 
 def export_observations_list_clicked(self):
@@ -1459,7 +1460,8 @@ def initialize_new_media_observation(self):
     # store state behaviors for subject current state
     self.state_behaviors_codes = tuple(util.state_behavior_codes(self.pj[cfg.ETHOGRAM]))
 
-    self.lbSpeed.setText(f"Player rate: x{self.play_rate:.3f}")
+    video_operations.display_play_rate(self)
+    """self.lbSpeed.setText(f"Player rate: <b>x{self.play_rate:.3f}</b>")"""
 
     # spectrogram
     if (
@@ -1836,3 +1838,29 @@ def initialize_new_images_observation(self):
     self.w_obs_info.setVisible(True)
 
     self.get_events_current_row()
+
+
+def event2media_file_name(observation: dict, timestamp: dec) -> Optional[str]:
+    """
+    returns the media file name correstiong to the event (start time in case of state event)
+
+    Returns:
+        str: name of media file containing the event
+    """
+
+    cumul_media_durations = [0]
+    for media_file in observation[cfg.FILE]["1"]:
+        media_duration = observation[cfg.MEDIA_INFO][cfg.LENGTH][media_file]
+        cumul_media_durations.append(cumul_media_durations[-1] + media_duration)
+
+    player_idx_list = [
+        idx for idx, x in enumerate(cumul_media_durations) if cumul_media_durations[idx - 1] < timestamp <= x
+    ]
+    if len(player_idx_list):
+        player_idx = player_idx_list[0] - 1
+        video_file_name = observation[cfg.FILE]["1"][player_idx]
+    else:
+        player_idx = -1
+        video_file_name = None
+
+    return video_file_name
