@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2022 Olivier Friard
+Copyright 2012-2023 Olivier Friard
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,7 +30,8 @@ from shutil import copyfile
 from typing import List, Tuple, Dict
 
 import tablib
-from PyQt5.QtWidgets import QMessageBox
+from PyQt5.QtWidgets import QMessageBox, QTableWidgetItem
+from PyQt5.QtCore import Qt
 
 from . import config as cfg
 from . import db_functions
@@ -38,6 +39,7 @@ from . import dialog
 from . import portion as I
 from . import utilities as util
 from . import version
+from . import observation_operations
 
 
 def check_observation_exhaustivity(
@@ -553,7 +555,7 @@ def create_subtitles(pj: dict, selected_observations: list, parameters: dict, ex
     for obs_id in selected_observations:
         if pj[cfg.OBSERVATIONS][obs_id][cfg.TYPE] == cfg.LIVE:
             out = ""
-            if parameters["time"] in ("limit to events", "full obs"):
+            if parameters["time"] in (cfg.TIME_EVENTS, cfg.TIME_FULL_OBS):
                 cursor.execute(
                     (
                         "SELECT subject, behavior, start, stop, type, modifiers FROM aggregated_events "
@@ -572,7 +574,7 @@ def create_subtitles(pj: dict, selected_observations: list, parameters: dict, ex
                     + parameters[cfg.SELECTED_BEHAVIORS],
                 )
 
-            else:  # 'time interval'
+            else:  # arbitrary 'time interval'
                 cursor.execute(
                     (
                         "SELECT subject, behavior, start, stop, type, modifiers FROM aggregated_events "
@@ -644,7 +646,7 @@ def create_subtitles(pj: dict, selected_observations: list, parameters: dict, ex
                         return False, f"The length for media file {media_file} is not available"
                     out = ""
 
-                    if parameters["time"] in ("limit to events", "full obs"):
+                    if parameters["time"] in (cfg.TIME_EVENTS, cfg.TIME_FULL_OBS):
                         cursor.execute(
                             (
                                 "SELECT subject, behavior, start, stop, type, modifiers FROM aggregated_events "
@@ -666,7 +668,7 @@ def create_subtitles(pj: dict, selected_observations: list, parameters: dict, ex
                             + parameters[cfg.SELECTED_BEHAVIORS],
                         )
 
-                    else:
+                    else:  # arbitrary 'time interval'
 
                         cursor.execute(
                             (
@@ -1522,3 +1524,86 @@ def has_audio(observation: dict, media_file_path: str) -> bool:
             if observation[cfg.MEDIA_INFO][cfg.HAS_AUDIO][media_file_path]:
                 return True
     return False
+
+
+def explore_project(self) -> None:
+    """
+    search various elements (subjects, behaviors, modifiers, comments) in all observations
+    """
+
+    def double_click_explore_project(obs_id, event_idx):
+        """
+        manage double-click on tablewidget of explore project results
+        """
+        observation_operations.load_observation(self, obs_id, cfg.VIEW)
+        self.twEvents.selectRow(event_idx - 1)
+        self.twEvents.scrollToItem(self.twEvents.item(event_idx - 1, 0))
+
+    elements_list = ("Subject", "Behavior", "Modifier", "Comment")
+    elements = []
+    for element in elements_list:
+        elements.append(("le", element))
+    elements.append(("cb", "Case sensitive", False))
+
+    explore_dlg = dialog.Input_dialog(
+        label_caption="Search in all observations", elements_list=elements, title="Explore project"
+    )
+    explore_dlg.pbOK.setText("Find")
+    if not explore_dlg.exec_():
+        return
+
+    nb_fields: int = 0
+    results: list = []
+    for element in elements_list:
+        nb_fields += explore_dlg.elements[element].text() != ""
+
+    for obs_id in sorted(self.pj[cfg.OBSERVATIONS]):
+        for event_idx, event in enumerate(self.pj[cfg.OBSERVATIONS][obs_id][cfg.EVENTS]):
+            nb_results = 0
+            for text, idx in (
+                (explore_dlg.elements["Subject"].text(), cfg.EVENT_SUBJECT_FIELD_IDX),
+                (explore_dlg.elements["Behavior"].text(), cfg.EVENT_BEHAVIOR_FIELD_IDX),
+                (explore_dlg.elements["Modifier"].text(), cfg.EVENT_MODIFIER_FIELD_IDX),
+                (explore_dlg.elements["Comment"].text(), cfg.EVENT_COMMENT_FIELD_IDX),
+            ):
+                if text:
+                    if any(
+                        (
+                            (explore_dlg.elements["Case sensitive"].isChecked() and text in event[idx]),
+                            (
+                                not explore_dlg.elements["Case sensitive"].isChecked()
+                                and text.upper() in event[idx].upper()
+                            ),
+                        )
+                    ):
+                        nb_results += 1
+
+            if nb_results == nb_fields:
+                results.append((obs_id, event_idx + 1))
+
+    if results:
+        self.results_dialog = dialog.View_explore_project_results()
+        self.results_dialog.setWindowTitle("Explore project results")
+        self.results_dialog.setWindowFlags(Qt.WindowStaysOnTopHint)
+        self.results_dialog.double_click_signal.connect(double_click_explore_project)
+        txt = f"<b>{len(results)}</b> events"
+        txt2 = ""
+        for element in elements_list:
+            if explore_dlg.elements[element].text():
+                txt2 += f"<b>{explore_dlg.elements[element].text()}</b> in {element}<br>"
+        if txt2:
+            txt += " for<br>"
+        self.results_dialog.lb.setText(txt + txt2)
+        self.results_dialog.tw.setColumnCount(2)
+        self.results_dialog.tw.setRowCount(len(results))
+        self.results_dialog.tw.setHorizontalHeaderLabels(["Observation id", "row index"])
+
+        for row, result in enumerate(results):
+            for i in range(0, 2):
+                self.results_dialog.tw.setItem(row, i, QTableWidgetItem(str(result[i])))
+                self.results_dialog.tw.item(row, i).setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
+
+        self.results_dialog.show()
+
+    else:
+        QMessageBox.information(self, cfg.programName, "No events found")
