@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2022 Olivier Friard
+Copyright 2012-2023 Olivier Friard
 
 This file is part of BORIS.
 
@@ -28,9 +28,9 @@ from boris.config import *
 import boris.utilities as utilities
 from boris import param_panel
 
-from PyQt5.QtWidgets import (QFileDialog, QTableWidgetItem, QApplication, QMessageBox, QListWidgetItem)
-from PyQt5.QtCore import (Qt)
-from PyQt5.QtGui import (QColor, QFont)
+from PyQt5.QtWidgets import QFileDialog, QTableWidgetItem, QApplication, QMessageBox, QListWidgetItem
+from PyQt5.QtCore import Qt
+from PyQt5.QtGui import QColor, QFont
 
 
 def check_text_file_type(rows):
@@ -53,13 +53,16 @@ def import_from_text_file(self):
 
     if self.twBehaviors.rowCount():
         response = dialog.MessageDialog(
-            programName, "There are behaviors already configured. Do you want to append behaviors or replace them?",
-            ['Append', 'Replace', CANCEL])
+            programName,
+            "There are behaviors already configured. Do you want to append behaviors or replace them?",
+            ["Append", "Replace", CANCEL],
+        )
         if response == CANCEL:
             return
 
-    fn = QFileDialog().getOpenFileName(self, "Import behaviors from text file", "",
-                                       "Text files (*.txt *.tsv *.csv);;All files (*)")
+    fn = QFileDialog().getOpenFileName(
+        self, "Import behaviors from text file", "", "Text files (*.txt *.tsv *.csv);;All files (*)"
+    )
     fileName = fn[0] if type(fn) is tuple else fn
 
     if fileName:
@@ -76,9 +79,16 @@ def import_from_text_file(self):
                 try:
                     rows.append(row.decode("utf-8"))
                 except Exception:
-                    QMessageBox.critical(None, programName, (f"Error while reading file\nThe line # {idx}\n"
-                                                             f"{row}\ncontains characters that are not readable."),
-                                         QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
+                    QMessageBox.critical(
+                        None,
+                        programName,
+                        (
+                            f"Error while reading file\nThe line # {idx}\n"
+                            f"{row}\ncontains characters that are not readable."
+                        ),
+                        QMessageBox.Ok | QMessageBox.Default,
+                        QMessageBox.NoButton,
+                    )
                     return
                 idx += 1
 
@@ -88,8 +98,10 @@ def import_from_text_file(self):
 
             if fieldSeparator is None:
                 QMessageBox.critical(
-                    self, programName,
-                    "Separator character not found! Use plain text file and TAB or comma as value separator")
+                    self,
+                    programName,
+                    "Separator character not found! Use plain text file and TAB or comma as value separator",
+                )
             else:
 
                 for row in rows:
@@ -112,7 +124,7 @@ def import_from_text_file(self):
                         "modifiers": "",
                         "excluded": "",
                         "coding map": "",
-                        "category": ""
+                        "category": "",
                     }
 
                     self.twBehaviors.setRowCount(self.twBehaviors.rowCount() + 1)
@@ -146,14 +158,17 @@ def import_behaviors_from_clipboard(self):
         cb = QApplication.clipboard()
         cb_text = cb.text()
         if not cb_text:
-            QMessageBox.warning(None, programName, "The clipboard is empty", QMessageBox.Ok | QMessageBox.Default,
-                                QMessageBox.NoButton)
+            QMessageBox.warning(
+                None, programName, "The clipboard is empty", QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton
+            )
             return
 
         if self.twBehaviors.rowCount():
             response = dialog.MessageDialog(
-                programName, "Some behaviors are already configured. Do you want to append behaviors or replace them?",
-                ["Append", "Replace", CANCEL])
+                programName,
+                "Some behaviors are already configured. Do you want to append behaviors or replace them?",
+                ["Append", "Replace", CANCEL],
+            )
             if response == CANCEL:
                 return
 
@@ -165,10 +180,16 @@ def import_behaviors_from_clipboard(self):
             cb_text_splitted.remove("")
 
         if len(set([len(x.split("\t")) for x in cb_text_splitted])) != 1:
-            QMessageBox.warning(None, programName,
-                                ("The clipboard content does not have a constant number of fields.<br>"
-                                 "From your spreadsheet: CTRL + A (select all cells), CTRL + C (copy to clipboard)"),
-                                QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
+            QMessageBox.warning(
+                None,
+                programName,
+                (
+                    "The clipboard content does not have a constant number of fields.<br>"
+                    "From your spreadsheet: CTRL + A (select all cells), CTRL + C (copy to clipboard)"
+                ),
+                QMessageBox.Ok | QMessageBox.Default,
+                QMessageBox.NoButton,
+            )
             return
 
         for row in cb_text_splitted:
@@ -176,8 +197,9 @@ def import_behaviors_from_clipboard(self):
                 behavior = {"type": DEFAULT_BEHAVIOR_TYPE}
                 for idx, field in enumerate(row.split("\t")):
                     if idx == 0:
-                        behavior["type"] = STATE_EVENT if STATE in field.upper() else (
-                            POINT_EVENT if POINT in field.upper() else "")
+                        behavior["type"] = (
+                            STATE_EVENT if STATE in field.upper() else (POINT_EVENT if POINT in field.upper() else "")
+                        )
                     if idx == 1:
                         behavior["key"] = field.strip() if len(field.strip()) == 1 else ""
                     if idx == 2:
@@ -195,7 +217,9 @@ def import_behaviors_from_clipboard(self):
                     else:
                         item = QTableWidgetItem(behavior.get(field_type, ""))
 
-                    if field_type not in ETHOGRAM_EDITABLE_FIELDS:  # [TYPE, "excluded", "coding map", "modifiers", "category"]:
+                    if (
+                        field_type not in ETHOGRAM_EDITABLE_FIELDS
+                    ):  # [TYPE, "excluded", "coding map", "modifiers", "category"]:
                         item.setFlags(Qt.ItemIsEnabled)
                         item.setBackground(QColor(230, 230, 230))
 
@@ -211,13 +235,16 @@ def import_from_JWatcher(self):
     try:
         if self.twBehaviors.rowCount():
             response = dialog.MessageDialog(
-                programName, "There are behaviors already configured. Do you want to append behaviors or replace them?",
-                ["Append", "Replace", CANCEL])
+                programName,
+                "There are behaviors already configured. Do you want to append behaviors or replace them?",
+                ["Append", "Replace", CANCEL],
+            )
             if response == CANCEL:
                 return
 
-        fn = QFileDialog().getOpenFileName(self, "Import behaviors from JWatcher", "",
-                                           "Global Definition File (*.gdf);;All files (*)")
+        fn = QFileDialog().getOpenFileName(
+            self, "Import behaviors from JWatcher", "", "Global Definition File (*.gdf);;All files (*)"
+        )
         fileName = fn[0] if type(fn) is tuple else fn
 
         if fileName:
@@ -232,7 +259,7 @@ def import_from_JWatcher(self):
                     continue
 
                 if "Behavior.name." in row and "=" in row:
-                    key, code = row.split('=')
+                    key, code = row.split("=")
                     key = key.replace("Behavior.name.", "")
                     # read description
                     if idx < len(rows) and "Behavior.description." in rows[idx + 1]:
@@ -245,7 +272,7 @@ def import_from_JWatcher(self):
                         "modifiers": "",
                         "excluded": "",
                         "coding map": "",
-                        "category": ""
+                        "category": "",
                     }
 
                     self.twBehaviors.setRowCount(self.twBehaviors.rowCount() + 1)
@@ -273,14 +300,17 @@ def import_subjects_from_clipboard(self):
         cb = QApplication.clipboard()
         cb_text = cb.text()
         if not cb_text:
-            QMessageBox.warning(None, programName, "The clipboard is empty", QMessageBox.Ok | QMessageBox.Default,
-                                QMessageBox.NoButton)
+            QMessageBox.warning(
+                None, programName, "The clipboard is empty", QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton
+            )
             return
 
         if self.twSubjects.rowCount():
             response = dialog.MessageDialog(
-                programName, "Some subjects are already configured. Do you want to append subjects or replace them?",
-                ["Append", "Replace", CANCEL])
+                programName,
+                "Some subjects are already configured. Do you want to append subjects or replace them?",
+                ["Append", "Replace", CANCEL],
+            )
             if response == CANCEL:
                 return
 
@@ -290,10 +320,16 @@ def import_subjects_from_clipboard(self):
         cb_text_splitted = cb_text.split("\n")
 
         if len(set([len(x.split("\t")) for x in cb_text_splitted])) != 1:
-            QMessageBox.warning(None, programName,
-                                ("The clipboard content does not have a constant number of fields.<br>"
-                                 "From your spreadsheet: CTRL + A (select all cells), CTRL + C (copy to clipboard)"),
-                                QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
+            QMessageBox.warning(
+                None,
+                programName,
+                (
+                    "The clipboard content does not have a constant number of fields.<br>"
+                    "From your spreadsheet: CTRL + A (select all cells), CTRL + C (copy to clipboard)"
+                ),
+                QMessageBox.Ok | QMessageBox.Default,
+                QMessageBox.NoButton,
+            )
             return
 
         for row in cb_text_splitted:
@@ -316,11 +352,13 @@ def import_subjects_from_clipboard(self):
         dialog.error_message(sys._getframe().f_code.co_name, sys.exc_info())
 
 
-def select_behaviors(title="Record value from external data file",
-                     text="Behaviors",
-                     behavioral_categories=[],
-                     ethogram={},
-                     behavior_type=[STATE_EVENT, POINT_EVENT]):
+def select_behaviors(
+    title="Record value from external data file",
+    text="Behaviors",
+    behavioral_categories=[],
+    ethogram={},
+    behavior_type=[STATE_EVENT, POINT_EVENT],
+):
     """
     allow user to select behaviors to import
 
@@ -338,10 +376,14 @@ def select_behaviors(title="Record value from external data file",
         paramPanelWindow.setWindowTitle(title)
         paramPanelWindow.lbBehaviors.setText(text)
         for w in [
-                paramPanelWindow.lwSubjects, paramPanelWindow.pbSelectAllSubjects,
-                paramPanelWindow.pbUnselectAllSubjects, paramPanelWindow.pbReverseSubjectsSelection,
-                paramPanelWindow.lbSubjects, paramPanelWindow.cbIncludeModifiers, paramPanelWindow.cbExcludeBehaviors,
-                paramPanelWindow.frm_time
+            paramPanelWindow.lwSubjects,
+            paramPanelWindow.pbSelectAllSubjects,
+            paramPanelWindow.pbUnselectAllSubjects,
+            paramPanelWindow.pbReverseSubjectsSelection,
+            paramPanelWindow.lbSubjects,
+            paramPanelWindow.cbIncludeModifiers,
+            paramPanelWindow.cbExcludeBehaviors,
+            paramPanelWindow.frm_time,
         ]:
             w.setVisible(False)
 
@@ -375,11 +417,14 @@ def select_behaviors(title="Record value from external data file",
             # check if behavior type must be shown
             for behavior in [ethogram[x][BEHAVIOR_CODE] for x in utilities.sorted_keys(ethogram)]:
 
-                if ((categories == ["###no category###"]) or (behavior in [
+                if (categories == ["###no category###"]) or (
+                    behavior
+                    in [
                         ethogram[x][BEHAVIOR_CODE]
                         for x in ethogram
                         if BEHAVIOR_CATEGORY in ethogram[x] and ethogram[x][BEHAVIOR_CATEGORY] == category
-                ])):
+                    ]
+                ):
 
                     paramPanelWindow.item = QListWidgetItem(behavior)
                     paramPanelWindow.item.setCheckState(Qt.Unchecked)
@@ -404,9 +449,9 @@ def select_behaviors(title="Record value from external data file",
 def import_behaviors_from_project(self):
     try:
 
-        fn = QFileDialog().getOpenFileName(self, "Import behaviors from project file", "",
-                                           ("Project files (*.boris *.boris.gz);;"
-                                            "All files (*)"))
+        fn = QFileDialog().getOpenFileName(
+            self, "Import behaviors from project file", "", ("Project files (*.boris *.boris.gz);;" "All files (*)")
+        )
         file_name = fn[0] if type(fn) is tuple else fn
 
         if file_name:
@@ -419,19 +464,23 @@ def import_behaviors_from_project(self):
             # configuration of behaviours
             if ETHOGRAM in project and project[ETHOGRAM]:
                 if self.twBehaviors.rowCount():
-                    response = dialog.MessageDialog(programName, ("Some behaviors are already configured. "
-                                                                  "Do you want to append behaviors or replace them?"),
-                                                    ["Append", "Replace", CANCEL])
+                    response = dialog.MessageDialog(
+                        programName,
+                        ("Some behaviors are already configured. " "Do you want to append behaviors or replace them?"),
+                        ["Append", "Replace", CANCEL],
+                    )
                     if response == "Replace":
                         self.twBehaviors.setRowCount(0)
                     if response == CANCEL:
                         return
 
-                behaviors_to_import = select_behaviors(title="Select the behaviors to import",
-                                                       text="Behaviors",
-                                                       behavioral_categories=list(project[BEHAVIORAL_CATEGORIES]),
-                                                       ethogram=dict(project[ETHOGRAM]),
-                                                       behavior_type=[STATE_EVENT, POINT_EVENT])
+                behaviors_to_import = select_behaviors(
+                    title="Select the behaviors to import",
+                    text="Behaviors",
+                    behavioral_categories=list(project[BEHAVIORAL_CATEGORIES]),
+                    ethogram=dict(project[ETHOGRAM]),
+                    behavior_type=[STATE_EVENT, POINT_EVENT],
+                )
 
                 for i in utilities.sorted_keys(project[ETHOGRAM]):
 
@@ -458,7 +507,7 @@ def import_behaviors_from_project(self):
                                         modif_set_dict[str(len(modif_set_dict))] = {
                                             "name": "",
                                             "type": SINGLE_SELECTION,
-                                            "values": modif_set.split(",")
+                                            "values": modif_set.split(","),
                                         }
                                 project[ETHOGRAM][i][field] = dict(modif_set_dict)
 
@@ -485,9 +534,9 @@ def import_subjects_from_project(self):
     """
 
     try:
-        fn = QFileDialog().getOpenFileName(self, "Import subjects from project file", "",
-                                           ("Project files (*.boris *.boris.gz);;"
-                                            "All files (*)"))
+        fn = QFileDialog().getOpenFileName(
+            self, "Import subjects from project file", "", ("Project files (*.boris *.boris.gz);;" "All files (*)")
+        )
         file_name = fn[0] if type(fn) is tuple else fn
 
         if file_name:
@@ -502,9 +551,11 @@ def import_subjects_from_project(self):
             if SUBJECTS in project and project[SUBJECTS]:
 
                 if self.twSubjects.rowCount():
-                    response = dialog.MessageDialog(programName, ("There are subjects already configured. "
-                                                                  "Do you want to append subjects or replace them?"),
-                                                    ['Append', 'Replace', 'Cancel'])
+                    response = dialog.MessageDialog(
+                        programName,
+                        ("There are subjects already configured. " "Do you want to append subjects or replace them?"),
+                        ["Append", "Replace", "Cancel"],
+                    )
 
                     if response == "Replace":
                         self.twSubjects.setRowCount(0)
@@ -519,8 +570,9 @@ def import_subjects_from_project(self):
                     for idx2, sbjField in enumerate(subjectsFields):
 
                         if sbjField in project[SUBJECTS][idx]:
-                            self.twSubjects.setItem(self.twSubjects.rowCount() - 1, idx2,
-                                                    QTableWidgetItem(project[SUBJECTS][idx][sbjField]))
+                            self.twSubjects.setItem(
+                                self.twSubjects.rowCount() - 1, idx2, QTableWidgetItem(project[SUBJECTS][idx][sbjField])
+                            )
                         else:
                             self.twSubjects.setItem(self.twSubjects.rowCount() - 1, idx2, QTableWidgetItem(""))
 
@@ -537,9 +589,12 @@ def import_indep_variables_from_project(self):
     """
 
     try:
-        fn = QFileDialog().getOpenFileName(self, "Import independent variables from project file", "",
-                                           ("Project files (*.boris *.boris.gz);;"
-                                            "All files (*)"))
+        fn = QFileDialog().getOpenFileName(
+            self,
+            "Import independent variables from project file",
+            "",
+            ("Project files (*.boris *.boris.gz);;" "All files (*)"),
+        )
 
         file_name = fn[0] if type(fn) is tuple else fn
 
