@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2022 Olivier Friard
+Copyright 2012-2023 Olivier Friard
 
 This file is part of BORIS.
 
@@ -391,8 +391,12 @@ def time_budget(self, mode: str, mode2: str = "list"):
         mode2 (str): must be in ["list", "current"]
     """
 
-    if mode2 == "current" and self.observationId:
-        selected_observations = [self.observationId]
+    if mode2 == "current":
+        if self.observationId:
+            selected_observations = [self.observationId]
+        else:
+            return
+
     if mode2 == "list":
         _, selected_observations = select_observations.select_observations(self.pj, mode=cfg.MULTIPLE, windows_title="")
 
@@ -436,6 +440,8 @@ def time_budget(self, mode: str, mode2: str = "list"):
         by_category=(mode == "by_category"),
         n_observations=len(selected_observations),
     )
+    if parameters == {}:
+        return
 
     if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
         QMessageBox.warning(None, cfg.programName, "Select subject(s) and behavior(s) to analyze")
