@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2022 Olivier Friard
+Copyright 2012-2023 Olivier Friard
 
 This file is part of BORIS.
 
@@ -181,7 +181,7 @@ def transitions_matrix(self, mode):
     if not selected_observations:
         return
 
-    plot_parameters = select_subj_behav.choose_obs_subj_behav_category(
+    parameters = select_subj_behav.choose_obs_subj_behav_category(
         self,
         selected_observations,
         flagShowIncludeModifiers=True,
@@ -189,11 +189,14 @@ def transitions_matrix(self, mode):
         n_observations=len(selected_observations),
     )
 
-    if not plot_parameters[cfg.SELECTED_SUBJECTS] or not plot_parameters[cfg.SELECTED_BEHAVIORS]:
+    if parameters == {}:
+        return
+
+    if not parameters[cfg.SELECTED_SUBJECTS] or not parameters[cfg.SELECTED_BEHAVIORS]:
         return
 
     flagMulti = False
-    if len(plot_parameters[cfg.SELECTED_SUBJECTS]) == 1:
+    if len(parameters[cfg.SELECTED_SUBJECTS]) == 1:
 
         fn = QFileDialog().getSaveFileName(
             None,
@@ -215,22 +218,22 @@ def transitions_matrix(self, mode):
         flagMulti = True
 
     flag_overwrite_all = False
-    for subject in plot_parameters[cfg.SELECTED_SUBJECTS]:
+    for subject in parameters[cfg.SELECTED_SUBJECTS]:
 
         logging.debug(f"subjects: {subject}")
 
         strings_list = []
-        for obsId in selected_observations:
+        for obs_id in selected_observations:
             strings_list.append(
                 export_observation.events_to_behavioral_sequences(
-                    self.pj, obsId, subject, plot_parameters, self.behav_seq_separator
+                    self.pj, obs_id, subject, parameters, self.behav_seq_separator
                 )
             )
 
         sequences, observed_behaviors = behavioral_strings_analysis(strings_list, self.behav_seq_separator)
 
         observed_matrix = observed_transitions_matrix(
-            sequences, sorted(list(set(observed_behaviors + plot_parameters[cfg.SELECTED_BEHAVIORS]))), mode=mode
+            sequences, sorted(list(set(observed_behaviors + parameters[cfg.SELECTED_BEHAVIORS]))), mode=mode
         )
 
         if not observed_matrix:
